// WebSMSRelay
// Copyright (c) 2014 by Haute Solutions / Ed Van Every (www.haute-solutions.com)
//
// This file is free software; you can redistribute it and/or modify
// it under the terms of either the GNU General Public License version 2
// or the GNU Lesser General Public License version 2.1, both as
// published by the Free Software Foundation.
//
// This program will run a simple Web Server on the EGGBoard which accepts a cell phone number and a short message.
// The EGGBoard will then send the message to the designated cell phone as an SMS message.
// Simply point a browser at the address obtained by the EGGBoard (vis DHCP) and complete the form data.
// Use the Serial Monitor to determine what IP address was provided to the EGGBoard via DHCP if necessary.

#include <SPI.h>
#include <Ethernet.h>

#define HW_SERIAL
#define GSM_ON 6                 // SIM900 PwrOn Pin
#define GSMBUFFSIZE 200

char GSMCommBuff[GSMBUFFSIZE];   // GSM Communications Buffer (Public)
char GSMTempBuff[GSMBUFFSIZE];   // GSM Work Buffer (Private)

// MAC Address for the EGGBoard
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };

// TCP Port to Listen on is specified here
EthernetServer server(80);

String RequestString = String(100);
String WorkString = String(100);

String CPN = String();
String MSG = String();

void setup(){
  Ethernet.begin(mac);
  Serial.begin(9600);
  
  GSMInit(9600);
  GSMSetParams();

  Serial.print("EGGBoard is at ");
  Serial.println(Ethernet.localIP());
}

void loop(){

  EthernetClient client = server.available();

  if (client) {
    while (client.connected()) {

      if (client.available()) {
        
        //Read char by char, echo to console, assemble to string, 
        char c = client.read();
        Serial.print(c);
        if (RequestString.length() < 100) RequestString += c;

        //if HTTP request has ended, process the string
        if (c == '\n') {

          ProcessCommand(RequestString);

          SendHTMLPage(client);
          
          RequestString="";
          client.stop();
        }
      }
    }
  }
}

int SendHTMLPage(EthernetClient client){
  client.println("HTTP/1.1 200 OK");
  client.println("Content-Type: text/html");
  client.println();

  client.println("<h1>EGGBoard Ethernet SMS Relay</h1>"); 
  client.println();
 
  client.print("<font color='normal'><form method=get>");
  client.print("Cell Phone Number: <input type=text name=CPN maxlength=20 value=\"");
  client.print(CPN);
  client.print("\"><br>");
    
  client.print("<font color='normal'><form method=get>");
  client.print("Message: <input type=text name=MSG maxlength=20 value=\"");
  client.print(MSG);
  client.print("\"><br><br>");
  
  client.println("<input type=submit value=SUBMIT></form>");
  
  client.println("</body></html>");
}

int ProcessCommand(String command) {
char number[21];
char message[41];
 
  int cpn=command.indexOf("?CPN=");
  int msg=command.indexOf("&MSG=");
  int hdr=command.indexOf(" HTTP/1.1");
 
  if (cpn < 0) return 0;
  
  CPN = FormCharConvert(command.substring(cpn+5,msg));
  MSG = FormCharConvert(command.substring(msg+5,hdr)); 
  
  Serial.println(CPN);
  Serial.println(MSG);
  Serial.println();
  
  CPN.toCharArray(number, 21);
  MSG.toCharArray(message, 41);

  GSMSendSMS(number, message);
    
}

String FormCharConvert(String HFC) {
int i,j;

  WorkString = "";
  
  for(i=0; i < HFC.length(); i++) {
    
    switch (HFC[i]) {
 
      case '+':
        WorkString += ' ';
        break;
 
      case '%':
        WorkString += HexToAscii(HFC[i+1], HFC[i+2]);
        i += 2;
        break;
 
      default:
        WorkString += HFC[i];
    }
    
  }

  return WorkString;
} 

char HexToAscii(char first, char second)
{
char hex[5], *stop;

  hex[0] = '0';
  hex[1] = 'x';
  hex[2] = first;
  hex[3] = second;
  hex[4] = 0;

  return strtol(hex, &stop, 16);
}

// -----------------------------------------------------------------------------------------------------------------------------
// ----------------------------- Here is the "Inline" GSMSMS Library (.cpp) ----------------------------------------------------
// --------------------------------- (Beyond Here Be Dragons!) -----------------------------------------------------------------
// -----------------------------------------------------------------------------------------------------------------------------

int GSMSendCmd(char *AT_cmd_string)
{
  return GSMSendCmdParams(AT_cmd_string, 1000, 1000, "OK", 10);
}

int GSMSendCmd(const __FlashStringHelper *AT_cmd_string)
{
  return GSMSendCmdParams(AT_cmd_string, 1000, 1000, "OK", 10);
}

int GSMSendCmdParams(char *AT_cmd_string, int start_comm_tmout, int max_interchar_tmout, 
                     char *response_string, int no_of_attempts) 
{  
int i;
int stat;

  for (i = 0; i < no_of_attempts; i++) {
    if (i > 0) delay(500); 

    GSMPrintln(AT_cmd_string);
    stat = GSMWaitResp(start_comm_tmout, max_interchar_tmout, response_string);
    if(stat < 2) return stat;
  }
  
  return 2;
}

int GSMSendCmdParams(const __FlashStringHelper *AT_cmd_string, int start_comm_tmout, int max_interchar_tmout, 
                     char *response_string, int no_of_attempts) 
{  
int i;
int stat;

  for (i = 0; i < no_of_attempts; i++) {
    if (i > 0) delay(500); 

    GSMPrintln(AT_cmd_string);
    stat = GSMWaitResp(start_comm_tmout, max_interchar_tmout, response_string);
    if(stat < 2) return stat;
  }
  
  return 2;
}

int GSMWaitResp(int start_comm_tmout, int max_interchar_tmout, char *response_string)
{
unsigned long prev_time;
int bytes_rcvd;
int bytes_avail;

    prev_time = millis();
    GSMCommBuff[0] = 0x00;
    bytes_rcvd = 0;
    do {
      bytes_avail = GSMAvailable();
      if(bytes_avail == 0) {
        if(bytes_rcvd == 0) {
          if ((unsigned long)(millis() - prev_time) >= start_comm_tmout) break;
        } else {
          if ((unsigned long)(millis() - prev_time) >= max_interchar_tmout) break;
        }
      } else {

        prev_time = millis();
      
        while(bytes_avail) {
          bytes_avail--;
          if(bytes_rcvd < (GSMBUFFSIZE-1)) {
            GSMCommBuff[bytes_rcvd] = GSMRead();
            GSMCommBuff[++bytes_rcvd] = '\0'; 
          } else GSMRead();      
        }
        
      }      
    } while(1);

    if(bytes_rcvd > 0) {
      if(strstr(GSMCommBuff,response_string) != NULL) return 0;
      else return 1;
    }
    
    return 2;    
}

int GSMInit(long baud_rate){
int i;
int cmdstat;
int bauds[8]={1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200};
 
  pinMode(GSM_ON, OUTPUT);

  GSMBegin(9600);
  
  // Try three times to turn on Module	
  for(i=0; i<3; i++){
    cmdstat=GSMSendCmd("AT");  // 0="OK", 1=Something Else (Bad Baud?), 2=No Response
    if(cmdstat == 2) {
	digitalWrite(GSM_ON, HIGH);
	delay(1200);
	digitalWrite(GSM_ON, LOW);
	delay(10000);
    } else break;
  }
  
  // If its responding at 9600 and thats our target baud rate we are done!	
  if(GSMSendCmd("AT") == 0){
    if(baud_rate = 9600) return 0; 
  }
 
  // Roll through the potential baud rates and try to reset to 9600 so we can talk with it
  for(i=0; i<8; i++){
    GSMBegin(bauds[i]);
    GSMPrintln(F("AT+IPR=9600"));
  }
  
  // Check to see if its NOW responding at 9600
  GSMBegin(9600);
  if(GSMSendCmd("AT") > 0) return 1;  // Won't Reply
  
  // If thats our target baud we are done!
  if(baud_rate = 9600) return 0;
 
  // Set Module to desired baud rate
  GSMPrint(F("AT+IPR="));
  GSMPrint(baud_rate);
  GSMPrint("\r");
  delay(500);
  
  // Check if we got it! 
  GSMBegin(baud_rate);
  delay(100);
  if(GSMSendCmd("AT") == 0) return 0;

  // Otherwise we have failed...
  return 2;
}

void GSMSetParams() {
  GSMSendCmdParams(F("AT&F"), 1000, 50, "OK", 5);                                  // Reset to the factory settings    
  GSMSendCmdParams(F("ATE0"), 500, 50, "OK", 5);                                   // switch off echo
  GSMSendCmdParams(F("AT+CLIP=1"), 500, 50, "OK", 5);                              // Request calling line identification
  GSMSendCmdParams(F("AT+CMEE=0"), 500, 50, "OK", 5);		                // Mobile Equipment Error Code
  GSMSendCmdParams(F("AT+CMGF=1"), 500, 50, "OK", 5);		                // set the SMS mode to text 
  GSMSendCmdParams(F("AT+CPBS=\"SM\""), 1000, 50, "OK", 5);                        // select phonebook memory storage
  GSMSendCmdParams(F("AT+CIPSHUT"), 500, 50, "SHUT OK", 5);
  
  GSMSendCmdParams(F("AT+CNMI=0"), 1000, 50, "OK", 2);                             // Disable messages about new SMS from the GSM module 
  GSMSendCmdParams(F("AT+CPMS=\"SM\",\"SM\",\"SM\""), 1000, 1000, "+CPMS:", 10);   // Init memory for SMS in the SIM card
  
  GSMSendCmdParams(F("ATE0"), 500, 50, "OK", 5);                                   // Turn Off Echo
  GSMSendCmdParams(F("AT+CLTS=1"), 500, 50, "OK", 5);                              // Use Network Time

//GSMSendCmdParams(F("AT+IPR=9600"), 500, 50, "OK", 5);		                // setup fixed baud rate
//GSMSendCmdParams(F("AT#SELINT=1"), 500, 50, "OK", 5);		                // setup mode
//GSMSendCmdParams(F("AT#GPIO=8,1,1"), 500, 50, "OK", 5);		                // Switch ON User LED - just as signalization we are here
//GSMSendCmdParams(F("AT#GPIO=9,0,0"), 500, 50, "OK", 5);		                // Sets GPIO9 as an input = user button
//GSMSendCmdParams(F("AT#GPIO=5,0,2"), 500, 50, "OK", 5);		                // allow audio amplifier control
//GSMSendCmdParams(F("AT#GPIO=8,0,1"), 500, 50, "OK", 5);		                // Switch OFF User LED- just as signalization we are finished
//GSMSendCmdParams(F("AT#SHFEC=1"), 500, 50, "OK", 5);		                // Echo canceller enabled 
//GSMSendCmdParams(F("AT#SRS=26,0"), 500, 50, "OK", 5);		                // Ringer tone select (0 to 32)
//GSMSendCmdParams(F("AT#HFMICG=7"), 1000, 50, "OK", 5);		                // Microphone gain (0 to 7) - response here sometimes takes more than 500msec. so 1000msec. is more safety		
//GSMSendCmdParams(F("ATS0=1"), 500, 50, "OK", 5);		                        // Auto answer after first ring enabled - auto answer is not used
//GSMSendCmdParams(F("AT#SRP=1"), 500, 50, "OK", 5);		                // select ringer path to handsfree
//GSMSendCmdParams(F("AT+CRSL=2"), 500, 50, "OK", 5);		                // select ringer sound level  
}

int GSMGetSMS(char *number, char *msg, int nsize, int msize) 
{
char *cptr;
int msgno;
int i;
  
  if(GSMSendCmd(F("AT+CMGL=\"REC UNREAD\"")) > 0) return 0;
  
  cptr=strstr(GSMCommBuff,"+CMGL:");
  if(cptr == NULL) return 0;
  msgno=atoi(cptr+6);    
  
  GSMPrint(F("AT+CMGR="));
  GSMPrintln(msgno);
  if(GSMWaitResp(1000, 1000, "OK") > 0) return 0;
  
  // CALLER will be between THIRD Parenthesis and FOURTH Parenthesis
  cptr=strstr(GSMCommBuff,"+CMGR:");
  for(i=0; i<3; i++) {
    cptr = strchr(cptr,0x22);
    cptr++;
  }
  strcpy(GSMTempBuff,cptr);
  cptr=strchr(GSMTempBuff,0x22);
  *cptr='\0';

  strncpy(number,GSMTempBuff,nsize);
  number[nsize-1]='\0';

  // MESSAGE will be on the following line
  cptr=strstr(GSMCommBuff,"+CMGR:");
  cptr=strchr(cptr,0x0A);
  if(cptr != NULL) {
    cptr++;
    strcpy(GSMTempBuff,cptr);
    cptr=strchr(GSMTempBuff,0x0D);
    if(cptr !=NULL ) {
      *cptr='\0';
      strncpy(msg,GSMTempBuff,msize);
      msg[msize-1]='\0';
    } else return 0;
  } else return 0;

  GSMPrint(F("AT+CMGD="));
  GSMPrintln(msgno);
  GSMWaitResp(1000, 1000, "OK");
  
  return msgno;
}

int GSMSendSMS(char *number, char *msg)
{
char msgend[2]={0x1a,'\0'};

  if(strlen(number) > GSMBUFFSIZE - 11) return 1;
  if(strlen(msg) > GSMBUFFSIZE -2) return 2;
  
  if(GSMPrepToSendSMS(number) > 0) return 3;
  GSMPrint(msg);
  GSMPrintln(msgend);
  if(GSMFlushSMS() > 0) return 4;  
  return 0;
}

int GSMSendSMS(char *number, const __FlashStringHelper *msg)
{
char msgend[2]={0x1a,'\0'};

  if(strlen(number) > GSMBUFFSIZE - 11) return 1;
  if(strlen((const prog_char*)number) > GSMBUFFSIZE -2) return 2;
  
  if(GSMPrepToSendSMS(number) > 0) return 3;
  GSMPrint(msg);
  GSMPrintln(msgend);
  if(GSMFlushSMS() > 0) return 4;  
  return 0;
}

int GSMPrepToSendSMS(char *number)
{
  GSMPrint(F("AT+CMGS=\""));
  GSMPrint(number);
  GSMPrintln("\"");
  
  return GSMWaitResp(1000,500,">");
}

int GSMFlushSMS()
{
    return GSMWaitResp(7000,5000,"+CMGS");
}

// ----------------------------- GSMSMS Library Routines with Hardware Dependencies --------------------------------------------

void GSMBegin(int Baud_Rate)
{
#ifdef HW_SERIAL
  return Serial2.begin(Baud_Rate);
#else
  return GSMSS.begin(Baud_Rate);
#endif
}

int GSMAvailable()
{
#ifdef HW_SERIAL
  return Serial2.available();
#else
  return GSMSS.available();
#endif
}

int GSMRead()
{
#ifdef HW_SERIAL
  return Serial2.read();
#else
  return GSMSS.read();
#endif
}

long GSMPrint(char *text)
{
#ifdef HW_SERIAL
  return Serial2.print(text);
#else
  return GSMSS.print(text);
#endif
}

long GSMPrint(const __FlashStringHelper *text)
{
#ifdef HW_SERIAL
  return Serial2.print(text);
#else
  return GSMSS.print(text);
#endif
}

long GSMPrint(int i)
{
#ifdef HW_SERIAL
  return Serial2.print(i);
#else
  return GSMSS.print(i);
#endif
}

long GSMPrintln(char *text)
{
#ifdef HW_SERIAL
  return Serial2.println(text);
#else
  return GSMSS.println(text);
#endif
}

long GSMPrintln(const __FlashStringHelper *text)
{
#ifdef HW_SERIAL
  return Serial2.println(text);
#else
  return GSMSS.println(text);
#endif
}

long GSMPrintln(int i)
{
#ifdef HW_SERIAL
  return Serial2.println(i);
#else
  return GSMSS.println(i);
#endif
}


